from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Attachment")


@_attrs_define
class Attachment:
    """
    Attributes:
        duid (str):
        order (str):
        name (str):
        kind (str):
        file_url (str):
        color_hex (str):
        recommendation_duid (Union[None, str]):
    """

    duid: str
    order: str
    name: str
    kind: str
    file_url: str
    color_hex: str
    recommendation_duid: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        order = self.order

        name = self.name

        kind = self.kind

        file_url = self.file_url

        color_hex = self.color_hex

        recommendation_duid: Union[None, str]
        recommendation_duid = self.recommendation_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "order": order,
                "name": name,
                "kind": kind,
                "fileUrl": file_url,
                "colorHex": color_hex,
                "recommendationDuid": recommendation_duid,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        order = d.pop("order")

        name = d.pop("name")

        kind = d.pop("kind")

        file_url = d.pop("fileUrl")

        color_hex = d.pop("colorHex")

        def _parse_recommendation_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        recommendation_duid = _parse_recommendation_duid(d.pop("recommendationDuid"))

        attachment = cls(
            duid=duid,
            order=order,
            name=name,
            kind=kind,
            file_url=file_url,
            color_hex=color_hex,
            recommendation_duid=recommendation_duid,
        )

        attachment.additional_properties = d
        return attachment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.icon_kind import IconKind
from ..models.sprint_mode import SprintMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="SpaceCreate")


@_attrs_define
class SpaceCreate:
    """
    Attributes:
        duid (str):
        order (str):
        drafter_duid (Union[None, Unset, str]):
        accessible_by_team (Union[Unset, bool]):
        accessible_by_user_duids (Union[Unset, List[str]]):
        title (Union[Unset, str]):
        abrev (Union[Unset, str]):
        description (Union[Unset, str]):
        icon_kind (Union[Unset, IconKind]): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (Union[Unset, str]):
        color_hex (Union[Unset, str]):
        sprint_mode (Union[Unset, SprintMode]): * `None` - NONE
            * `ANBA` - ANBA
        sprint_replicate_on_rollover (Union[Unset, bool]):
        sprint_name_fmt (Union[Unset, str]):
        standup_recurrence (Union[Any, None, Unset]):
        standup_recurs_next_at (Union[None, Unset, datetime.datetime]):
        changelog_recurrence (Union[Any, None, Unset]):
        changelog_recurs_next_at (Union[None, Unset, datetime.datetime]):
    """

    duid: str
    order: str
    drafter_duid: Union[None, Unset, str] = UNSET
    accessible_by_team: Union[Unset, bool] = UNSET
    accessible_by_user_duids: Union[Unset, List[str]] = UNSET
    title: Union[Unset, str] = UNSET
    abrev: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    icon_kind: Union[Unset, IconKind] = UNSET
    icon_name_or_emoji: Union[Unset, str] = UNSET
    color_hex: Union[Unset, str] = UNSET
    sprint_mode: Union[Unset, SprintMode] = UNSET
    sprint_replicate_on_rollover: Union[Unset, bool] = UNSET
    sprint_name_fmt: Union[Unset, str] = UNSET
    standup_recurrence: Union[Any, None, Unset] = UNSET
    standup_recurs_next_at: Union[None, Unset, datetime.datetime] = UNSET
    changelog_recurrence: Union[Any, None, Unset] = UNSET
    changelog_recurs_next_at: Union[None, Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        order = self.order

        drafter_duid: Union[None, Unset, str]
        if isinstance(self.drafter_duid, Unset):
            drafter_duid = UNSET
        else:
            drafter_duid = self.drafter_duid

        accessible_by_team = self.accessible_by_team

        accessible_by_user_duids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.accessible_by_user_duids, Unset):
            accessible_by_user_duids = self.accessible_by_user_duids

        title = self.title

        abrev = self.abrev

        description = self.description

        icon_kind: Union[Unset, str] = UNSET
        if not isinstance(self.icon_kind, Unset):
            icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        sprint_mode: Union[Unset, str] = UNSET
        if not isinstance(self.sprint_mode, Unset):
            sprint_mode = self.sprint_mode.value

        sprint_replicate_on_rollover = self.sprint_replicate_on_rollover

        sprint_name_fmt = self.sprint_name_fmt

        standup_recurrence: Union[Any, None, Unset]
        if isinstance(self.standup_recurrence, Unset):
            standup_recurrence = UNSET
        else:
            standup_recurrence = self.standup_recurrence

        standup_recurs_next_at: Union[None, Unset, str]
        if isinstance(self.standup_recurs_next_at, Unset):
            standup_recurs_next_at = UNSET
        elif isinstance(self.standup_recurs_next_at, datetime.datetime):
            standup_recurs_next_at = self.standup_recurs_next_at.isoformat()
        else:
            standup_recurs_next_at = self.standup_recurs_next_at

        changelog_recurrence: Union[Any, None, Unset]
        if isinstance(self.changelog_recurrence, Unset):
            changelog_recurrence = UNSET
        else:
            changelog_recurrence = self.changelog_recurrence

        changelog_recurs_next_at: Union[None, Unset, str]
        if isinstance(self.changelog_recurs_next_at, Unset):
            changelog_recurs_next_at = UNSET
        elif isinstance(self.changelog_recurs_next_at, datetime.datetime):
            changelog_recurs_next_at = self.changelog_recurs_next_at.isoformat()
        else:
            changelog_recurs_next_at = self.changelog_recurs_next_at

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "order": order,
            }
        )
        if drafter_duid is not UNSET:
            field_dict["drafterDuid"] = drafter_duid
        if accessible_by_team is not UNSET:
            field_dict["accessibleByTeam"] = accessible_by_team
        if accessible_by_user_duids is not UNSET:
            field_dict["accessibleByUserDuids"] = accessible_by_user_duids
        if title is not UNSET:
            field_dict["title"] = title
        if abrev is not UNSET:
            field_dict["abrev"] = abrev
        if description is not UNSET:
            field_dict["description"] = description
        if icon_kind is not UNSET:
            field_dict["iconKind"] = icon_kind
        if icon_name_or_emoji is not UNSET:
            field_dict["iconNameOrEmoji"] = icon_name_or_emoji
        if color_hex is not UNSET:
            field_dict["colorHex"] = color_hex
        if sprint_mode is not UNSET:
            field_dict["sprintMode"] = sprint_mode
        if sprint_replicate_on_rollover is not UNSET:
            field_dict["sprintReplicateOnRollover"] = sprint_replicate_on_rollover
        if sprint_name_fmt is not UNSET:
            field_dict["sprintNameFmt"] = sprint_name_fmt
        if standup_recurrence is not UNSET:
            field_dict["standupRecurrence"] = standup_recurrence
        if standup_recurs_next_at is not UNSET:
            field_dict["standupRecursNextAt"] = standup_recurs_next_at
        if changelog_recurrence is not UNSET:
            field_dict["changelogRecurrence"] = changelog_recurrence
        if changelog_recurs_next_at is not UNSET:
            field_dict["changelogRecursNextAt"] = changelog_recurs_next_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        order = d.pop("order")

        def _parse_drafter_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        drafter_duid = _parse_drafter_duid(d.pop("drafterDuid", UNSET))

        accessible_by_team = d.pop("accessibleByTeam", UNSET)

        accessible_by_user_duids = cast(List[str], d.pop("accessibleByUserDuids", UNSET))

        title = d.pop("title", UNSET)

        abrev = d.pop("abrev", UNSET)

        description = d.pop("description", UNSET)

        _icon_kind = d.pop("iconKind", UNSET)
        icon_kind: Union[Unset, IconKind]
        if isinstance(_icon_kind, Unset):
            icon_kind = UNSET
        else:
            icon_kind = IconKind(_icon_kind)

        icon_name_or_emoji = d.pop("iconNameOrEmoji", UNSET)

        color_hex = d.pop("colorHex", UNSET)

        _sprint_mode = d.pop("sprintMode", UNSET)
        sprint_mode: Union[Unset, SprintMode]
        if isinstance(_sprint_mode, Unset):
            sprint_mode = UNSET
        else:
            sprint_mode = SprintMode(_sprint_mode)

        sprint_replicate_on_rollover = d.pop("sprintReplicateOnRollover", UNSET)

        sprint_name_fmt = d.pop("sprintNameFmt", UNSET)

        def _parse_standup_recurrence(data: object) -> Union[Any, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, None, Unset], data)

        standup_recurrence = _parse_standup_recurrence(d.pop("standupRecurrence", UNSET))

        def _parse_standup_recurs_next_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                standup_recurs_next_at_type_0 = isoparse(data)

                return standup_recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        standup_recurs_next_at = _parse_standup_recurs_next_at(d.pop("standupRecursNextAt", UNSET))

        def _parse_changelog_recurrence(data: object) -> Union[Any, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[Any, None, Unset], data)

        changelog_recurrence = _parse_changelog_recurrence(d.pop("changelogRecurrence", UNSET))

        def _parse_changelog_recurs_next_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                changelog_recurs_next_at_type_0 = isoparse(data)

                return changelog_recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        changelog_recurs_next_at = _parse_changelog_recurs_next_at(d.pop("changelogRecursNextAt", UNSET))

        space_create = cls(
            duid=duid,
            order=order,
            drafter_duid=drafter_duid,
            accessible_by_team=accessible_by_team,
            accessible_by_user_duids=accessible_by_user_duids,
            title=title,
            abrev=abrev,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
            sprint_mode=sprint_mode,
            sprint_replicate_on_rollover=sprint_replicate_on_rollover,
            sprint_name_fmt=sprint_name_fmt,
            standup_recurrence=standup_recurrence,
            standup_recurs_next_at=standup_recurs_next_at,
            changelog_recurrence=changelog_recurrence,
            changelog_recurs_next_at=changelog_recurs_next_at,
        )

        space_create.additional_properties = d
        return space_create

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.comment_reaction import CommentReaction


T = TypeVar("T", bound="Comment")


@_attrs_define
class Comment:
    """
    Attributes:
        duid (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        task_duid (str):
        root_duid (Union[None, str]):
        authored_by_ai (bool):
        author_duid (str):
        text (Any):
        published_at (Union[None, datetime.datetime]):
        reactions (List['CommentReaction']):
        is_draft (bool):
        edited (bool):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    task_duid: str
    root_duid: Union[None, str]
    authored_by_ai: bool
    author_duid: str
    text: Any
    published_at: Union[None, datetime.datetime]
    reactions: List["CommentReaction"]
    is_draft: bool
    edited: bool
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        task_duid = self.task_duid

        root_duid: Union[None, str]
        root_duid = self.root_duid

        authored_by_ai = self.authored_by_ai

        author_duid = self.author_duid

        text = self.text

        published_at: Union[None, str]
        if isinstance(self.published_at, datetime.datetime):
            published_at = self.published_at.isoformat()
        else:
            published_at = self.published_at

        reactions = []
        for reactions_item_data in self.reactions:
            reactions_item = reactions_item_data.to_dict()
            reactions.append(reactions_item)

        is_draft = self.is_draft

        edited = self.edited

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "taskDuid": task_duid,
                "rootDuid": root_duid,
                "authoredByAi": authored_by_ai,
                "authorDuid": author_duid,
                "text": text,
                "publishedAt": published_at,
                "reactions": reactions,
                "isDraft": is_draft,
                "edited": edited,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.comment_reaction import CommentReaction

        d = src_dict.copy()
        duid = d.pop("duid")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        task_duid = d.pop("taskDuid")

        def _parse_root_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        root_duid = _parse_root_duid(d.pop("rootDuid"))

        authored_by_ai = d.pop("authoredByAi")

        author_duid = d.pop("authorDuid")

        text = d.pop("text")

        def _parse_published_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                published_at_type_0 = isoparse(data)

                return published_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        published_at = _parse_published_at(d.pop("publishedAt"))

        reactions = []
        _reactions = d.pop("reactions")
        for reactions_item_data in _reactions:
            reactions_item = CommentReaction.from_dict(reactions_item_data)

            reactions.append(reactions_item)

        is_draft = d.pop("isDraft")

        edited = d.pop("edited")

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        comment = cls(
            duid=duid,
            created_at=created_at,
            updated_at=updated_at,
            task_duid=task_duid,
            root_duid=root_duid,
            authored_by_ai=authored_by_ai,
            author_duid=author_duid,
            text=text,
            published_at=published_at,
            reactions=reactions,
            is_draft=is_draft,
            edited=edited,
            updated_by_client_duid=updated_by_client_duid,
        )

        comment.additional_properties = d
        return comment

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_task_list import PaginatedTaskList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    assignee: Union[Unset, str] = UNSET,
    assignee_duid: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_duid: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.date] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    kind: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.date] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_duid: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["assignee"] = assignee

    params["assignee_duid"] = assignee_duid

    params["dartboard"] = dartboard

    params["dartboard_duid"] = dartboard_duid

    params["description"] = description

    json_due_at: Union[Unset, str] = UNSET
    if not isinstance(due_at, Unset):
        json_due_at = due_at.isoformat()
    params["due_at"] = json_due_at

    params["in_trash"] = in_trash

    params["is_draft"] = is_draft

    params["kind"] = kind

    params["limit"] = limit

    params["offset"] = offset

    params["priority"] = priority

    params["size"] = size

    json_start_at: Union[Unset, str] = UNSET
    if not isinstance(start_at, Unset):
        json_start_at = start_at.isoformat()
    params["start_at"] = json_start_at

    params["status"] = status

    params["status_duid"] = status_duid

    params["subscriber"] = subscriber

    params["subscriber_duid"] = subscriber_duid

    params["tag"] = tag

    params["title"] = title

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/api/v0/tasks",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[PaginatedTaskList]:
    if response.status_code == HTTPStatus.OK:
        response_200 = PaginatedTaskList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[PaginatedTaskList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    assignee: Union[Unset, str] = UNSET,
    assignee_duid: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_duid: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.date] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    kind: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.date] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_duid: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedTaskList]:
    """
    Args:
        assignee (Union[Unset, str]):
        assignee_duid (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_duid (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.date]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        kind (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.date]):
        status (Union[Unset, str]):
        status_duid (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        tag (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedTaskList]
    """

    kwargs = _get_kwargs(
        assignee=assignee,
        assignee_duid=assignee_duid,
        dartboard=dartboard,
        dartboard_duid=dartboard_duid,
        description=description,
        due_at=due_at,
        in_trash=in_trash,
        is_draft=is_draft,
        kind=kind,
        limit=limit,
        offset=offset,
        priority=priority,
        size=size,
        start_at=start_at,
        status=status,
        status_duid=status_duid,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        tag=tag,
        title=title,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    assignee: Union[Unset, str] = UNSET,
    assignee_duid: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_duid: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.date] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    kind: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.date] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_duid: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedTaskList]:
    """
    Args:
        assignee (Union[Unset, str]):
        assignee_duid (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_duid (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.date]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        kind (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.date]):
        status (Union[Unset, str]):
        status_duid (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        tag (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedTaskList
    """

    return sync_detailed(
        client=client,
        assignee=assignee,
        assignee_duid=assignee_duid,
        dartboard=dartboard,
        dartboard_duid=dartboard_duid,
        description=description,
        due_at=due_at,
        in_trash=in_trash,
        is_draft=is_draft,
        kind=kind,
        limit=limit,
        offset=offset,
        priority=priority,
        size=size,
        start_at=start_at,
        status=status,
        status_duid=status_duid,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        tag=tag,
        title=title,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    assignee: Union[Unset, str] = UNSET,
    assignee_duid: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_duid: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.date] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    kind: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.date] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_duid: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedTaskList]:
    """
    Args:
        assignee (Union[Unset, str]):
        assignee_duid (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_duid (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.date]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        kind (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.date]):
        status (Union[Unset, str]):
        status_duid (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        tag (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedTaskList]
    """

    kwargs = _get_kwargs(
        assignee=assignee,
        assignee_duid=assignee_duid,
        dartboard=dartboard,
        dartboard_duid=dartboard_duid,
        description=description,
        due_at=due_at,
        in_trash=in_trash,
        is_draft=is_draft,
        kind=kind,
        limit=limit,
        offset=offset,
        priority=priority,
        size=size,
        start_at=start_at,
        status=status,
        status_duid=status_duid,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        tag=tag,
        title=title,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    assignee: Union[Unset, str] = UNSET,
    assignee_duid: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_duid: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.date] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    kind: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.date] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_duid: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedTaskList]:
    """
    Args:
        assignee (Union[Unset, str]):
        assignee_duid (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_duid (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.date]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        kind (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.date]):
        status (Union[Unset, str]):
        status_duid (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        tag (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedTaskList
    """

    return (
        await asyncio_detailed(
            client=client,
            assignee=assignee,
            assignee_duid=assignee_duid,
            dartboard=dartboard,
            dartboard_duid=dartboard_duid,
            description=description,
            due_at=due_at,
            in_trash=in_trash,
            is_draft=is_draft,
            kind=kind,
            limit=limit,
            offset=offset,
            priority=priority,
            size=size,
            start_at=start_at,
            status=status,
            status_duid=status_duid,
            subscriber=subscriber,
            subscriber_duid=subscriber_duid,
            tag=tag,
            title=title,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.paginated_doc_list import PaginatedDocList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    editor: Union[Unset, str] = UNSET,
    editor_duid: Union[Unset, str] = UNSET,
    folder: Union[Unset, str] = UNSET,
    folder_duid: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    report_kind: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["editor"] = editor

    params["editor_duid"] = editor_duid

    params["folder"] = folder

    params["folder_duid"] = folder_duid

    params["in_trash"] = in_trash

    params["is_draft"] = is_draft

    params["limit"] = limit

    params["offset"] = offset

    params["report_kind"] = report_kind

    params["subscriber"] = subscriber

    params["subscriber_duid"] = subscriber_duid

    params["text"] = text

    params["title"] = title

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/api/v0/docs",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[PaginatedDocList]:
    if response.status_code == HTTPStatus.OK:
        response_200 = PaginatedDocList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[PaginatedDocList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    editor: Union[Unset, str] = UNSET,
    editor_duid: Union[Unset, str] = UNSET,
    folder: Union[Unset, str] = UNSET,
    folder_duid: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    report_kind: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedDocList]:
    """
    Args:
        editor (Union[Unset, str]):
        editor_duid (Union[Unset, str]):
        folder (Union[Unset, str]):
        folder_duid (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        report_kind (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedDocList]
    """

    kwargs = _get_kwargs(
        editor=editor,
        editor_duid=editor_duid,
        folder=folder,
        folder_duid=folder_duid,
        in_trash=in_trash,
        is_draft=is_draft,
        limit=limit,
        offset=offset,
        report_kind=report_kind,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        text=text,
        title=title,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    editor: Union[Unset, str] = UNSET,
    editor_duid: Union[Unset, str] = UNSET,
    folder: Union[Unset, str] = UNSET,
    folder_duid: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    report_kind: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedDocList]:
    """
    Args:
        editor (Union[Unset, str]):
        editor_duid (Union[Unset, str]):
        folder (Union[Unset, str]):
        folder_duid (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        report_kind (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedDocList
    """

    return sync_detailed(
        client=client,
        editor=editor,
        editor_duid=editor_duid,
        folder=folder,
        folder_duid=folder_duid,
        in_trash=in_trash,
        is_draft=is_draft,
        limit=limit,
        offset=offset,
        report_kind=report_kind,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        text=text,
        title=title,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    editor: Union[Unset, str] = UNSET,
    editor_duid: Union[Unset, str] = UNSET,
    folder: Union[Unset, str] = UNSET,
    folder_duid: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    report_kind: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Response[PaginatedDocList]:
    """
    Args:
        editor (Union[Unset, str]):
        editor_duid (Union[Unset, str]):
        folder (Union[Unset, str]):
        folder_duid (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        report_kind (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedDocList]
    """

    kwargs = _get_kwargs(
        editor=editor,
        editor_duid=editor_duid,
        folder=folder,
        folder_duid=folder_duid,
        in_trash=in_trash,
        is_draft=is_draft,
        limit=limit,
        offset=offset,
        report_kind=report_kind,
        subscriber=subscriber,
        subscriber_duid=subscriber_duid,
        text=text,
        title=title,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    editor: Union[Unset, str] = UNSET,
    editor_duid: Union[Unset, str] = UNSET,
    folder: Union[Unset, str] = UNSET,
    folder_duid: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_draft: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = UNSET,
    report_kind: Union[Unset, str] = UNSET,
    subscriber: Union[Unset, str] = UNSET,
    subscriber_duid: Union[Unset, str] = UNSET,
    text: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
) -> Optional[PaginatedDocList]:
    """
    Args:
        editor (Union[Unset, str]):
        editor_duid (Union[Unset, str]):
        folder (Union[Unset, str]):
        folder_duid (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_draft (Union[Unset, bool]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):
        report_kind (Union[Unset, str]):
        subscriber (Union[Unset, str]):
        subscriber_duid (Union[Unset, str]):
        text (Union[Unset, str]):
        title (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedDocList
    """

    return (
        await asyncio_detailed(
            client=client,
            editor=editor,
            editor_duid=editor_duid,
            folder=folder,
            folder_duid=folder_duid,
            in_trash=in_trash,
            is_draft=is_draft,
            limit=limit,
            offset=offset,
            report_kind=report_kind,
            subscriber=subscriber,
            subscriber_duid=subscriber_duid,
            text=text,
            title=title,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TenantUpdate")


@_attrs_define
class TenantUpdate:
    """
    Attributes:
        name (Union[Unset, str]):
        timezone (Union[Unset, str]):
        backlog_enabled (Union[Unset, bool]):
        ai_assignment_enabled (Union[Unset, bool]):
        email_integration_enabled (Union[Unset, bool]):
        close_parent_on_close_all_subtasks (Union[Unset, bool]):
        move_subtasks_on_move_parent (Union[Unset, bool]):
        update_subtasks_status_on_update_parent_status (Union[Unset, bool]):
        copy_parent_fields_on_create (Union[Unset, bool]):
        update_blockee_dates_on_update_blocker_due_date (Union[Unset, bool]):
        webhook_enabled (Union[Unset, bool]):
        webhook_secret (Union[Unset, str]):
    """

    name: Union[Unset, str] = UNSET
    timezone: Union[Unset, str] = UNSET
    backlog_enabled: Union[Unset, bool] = UNSET
    ai_assignment_enabled: Union[Unset, bool] = UNSET
    email_integration_enabled: Union[Unset, bool] = UNSET
    close_parent_on_close_all_subtasks: Union[Unset, bool] = UNSET
    move_subtasks_on_move_parent: Union[Unset, bool] = UNSET
    update_subtasks_status_on_update_parent_status: Union[Unset, bool] = UNSET
    copy_parent_fields_on_create: Union[Unset, bool] = UNSET
    update_blockee_dates_on_update_blocker_due_date: Union[Unset, bool] = UNSET
    webhook_enabled: Union[Unset, bool] = UNSET
    webhook_secret: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        timezone = self.timezone

        backlog_enabled = self.backlog_enabled

        ai_assignment_enabled = self.ai_assignment_enabled

        email_integration_enabled = self.email_integration_enabled

        close_parent_on_close_all_subtasks = self.close_parent_on_close_all_subtasks

        move_subtasks_on_move_parent = self.move_subtasks_on_move_parent

        update_subtasks_status_on_update_parent_status = self.update_subtasks_status_on_update_parent_status

        copy_parent_fields_on_create = self.copy_parent_fields_on_create

        update_blockee_dates_on_update_blocker_due_date = self.update_blockee_dates_on_update_blocker_due_date

        webhook_enabled = self.webhook_enabled

        webhook_secret = self.webhook_secret

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if backlog_enabled is not UNSET:
            field_dict["backlogEnabled"] = backlog_enabled
        if ai_assignment_enabled is not UNSET:
            field_dict["aiAssignmentEnabled"] = ai_assignment_enabled
        if email_integration_enabled is not UNSET:
            field_dict["emailIntegrationEnabled"] = email_integration_enabled
        if close_parent_on_close_all_subtasks is not UNSET:
            field_dict["closeParentOnCloseAllSubtasks"] = close_parent_on_close_all_subtasks
        if move_subtasks_on_move_parent is not UNSET:
            field_dict["moveSubtasksOnMoveParent"] = move_subtasks_on_move_parent
        if update_subtasks_status_on_update_parent_status is not UNSET:
            field_dict["updateSubtasksStatusOnUpdateParentStatus"] = update_subtasks_status_on_update_parent_status
        if copy_parent_fields_on_create is not UNSET:
            field_dict["copyParentFieldsOnCreate"] = copy_parent_fields_on_create
        if update_blockee_dates_on_update_blocker_due_date is not UNSET:
            field_dict["updateBlockeeDatesOnUpdateBlockerDueDate"] = update_blockee_dates_on_update_blocker_due_date
        if webhook_enabled is not UNSET:
            field_dict["webhookEnabled"] = webhook_enabled
        if webhook_secret is not UNSET:
            field_dict["webhookSecret"] = webhook_secret

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        timezone = d.pop("timezone", UNSET)

        backlog_enabled = d.pop("backlogEnabled", UNSET)

        ai_assignment_enabled = d.pop("aiAssignmentEnabled", UNSET)

        email_integration_enabled = d.pop("emailIntegrationEnabled", UNSET)

        close_parent_on_close_all_subtasks = d.pop("closeParentOnCloseAllSubtasks", UNSET)

        move_subtasks_on_move_parent = d.pop("moveSubtasksOnMoveParent", UNSET)

        update_subtasks_status_on_update_parent_status = d.pop("updateSubtasksStatusOnUpdateParentStatus", UNSET)

        copy_parent_fields_on_create = d.pop("copyParentFieldsOnCreate", UNSET)

        update_blockee_dates_on_update_blocker_due_date = d.pop("updateBlockeeDatesOnUpdateBlockerDueDate", UNSET)

        webhook_enabled = d.pop("webhookEnabled", UNSET)

        webhook_secret = d.pop("webhookSecret", UNSET)

        tenant_update = cls(
            name=name,
            timezone=timezone,
            backlog_enabled=backlog_enabled,
            ai_assignment_enabled=ai_assignment_enabled,
            email_integration_enabled=email_integration_enabled,
            close_parent_on_close_all_subtasks=close_parent_on_close_all_subtasks,
            move_subtasks_on_move_parent=move_subtasks_on_move_parent,
            update_subtasks_status_on_update_parent_status=update_subtasks_status_on_update_parent_status,
            copy_parent_fields_on_create=copy_parent_fields_on_create,
            update_blockee_dates_on_update_blocker_due_date=update_blockee_dates_on_update_blocker_due_date,
            webhook_enabled=webhook_enabled,
            webhook_secret=webhook_secret,
        )

        tenant_update.additional_properties = d
        return tenant_update

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.icon_kind import IconKind
from ..models.space_kind import SpaceKind
from ..models.sprint_mode import SprintMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="Space")


@_attrs_define
class Space:
    """
    Attributes:
        duid (str):
        drafter_duid (Union[None, str]):
        kind (SpaceKind): * `Other` - OTHER
            * `Workspace` - WORKSPACE
            * `Personal` - PERSONAL
        accessible_by_team (bool):
        accessible_by_user_duids (List[str]):
        order (str):
        title (str):
        abrev (str):
        description (str):
        icon_kind (IconKind): * `None` - NONE
            * `Icon` - ICON
            * `Emoji` - EMOJI
        icon_name_or_emoji (str):
        color_hex (str):
        sprint_mode (SprintMode): * `None` - NONE
            * `ANBA` - ANBA
        sprint_replicate_on_rollover (bool):
        sprint_name_fmt (str):
        standup_recurrence (Union[Any, None]):
        standup_recurs_next_at (Union[None, datetime.datetime]):
        changelog_recurrence (Union[Any, None]):
        changelog_recurs_next_at (Union[None, datetime.datetime]):
        updated_by_client_duid (Union[None, Unset, str]):
    """

    duid: str
    drafter_duid: Union[None, str]
    kind: SpaceKind
    accessible_by_team: bool
    accessible_by_user_duids: List[str]
    order: str
    title: str
    abrev: str
    description: str
    icon_kind: IconKind
    icon_name_or_emoji: str
    color_hex: str
    sprint_mode: SprintMode
    sprint_replicate_on_rollover: bool
    sprint_name_fmt: str
    standup_recurrence: Union[Any, None]
    standup_recurs_next_at: Union[None, datetime.datetime]
    changelog_recurrence: Union[Any, None]
    changelog_recurs_next_at: Union[None, datetime.datetime]
    updated_by_client_duid: Union[None, Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        duid = self.duid

        drafter_duid: Union[None, str]
        drafter_duid = self.drafter_duid

        kind = self.kind.value

        accessible_by_team = self.accessible_by_team

        accessible_by_user_duids = self.accessible_by_user_duids

        order = self.order

        title = self.title

        abrev = self.abrev

        description = self.description

        icon_kind = self.icon_kind.value

        icon_name_or_emoji = self.icon_name_or_emoji

        color_hex = self.color_hex

        sprint_mode = self.sprint_mode.value

        sprint_replicate_on_rollover = self.sprint_replicate_on_rollover

        sprint_name_fmt = self.sprint_name_fmt

        standup_recurrence: Union[Any, None]
        standup_recurrence = self.standup_recurrence

        standup_recurs_next_at: Union[None, str]
        if isinstance(self.standup_recurs_next_at, datetime.datetime):
            standup_recurs_next_at = self.standup_recurs_next_at.isoformat()
        else:
            standup_recurs_next_at = self.standup_recurs_next_at

        changelog_recurrence: Union[Any, None]
        changelog_recurrence = self.changelog_recurrence

        changelog_recurs_next_at: Union[None, str]
        if isinstance(self.changelog_recurs_next_at, datetime.datetime):
            changelog_recurs_next_at = self.changelog_recurs_next_at.isoformat()
        else:
            changelog_recurs_next_at = self.changelog_recurs_next_at

        updated_by_client_duid: Union[None, Unset, str]
        if isinstance(self.updated_by_client_duid, Unset):
            updated_by_client_duid = UNSET
        else:
            updated_by_client_duid = self.updated_by_client_duid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "duid": duid,
                "drafterDuid": drafter_duid,
                "kind": kind,
                "accessibleByTeam": accessible_by_team,
                "accessibleByUserDuids": accessible_by_user_duids,
                "order": order,
                "title": title,
                "abrev": abrev,
                "description": description,
                "iconKind": icon_kind,
                "iconNameOrEmoji": icon_name_or_emoji,
                "colorHex": color_hex,
                "sprintMode": sprint_mode,
                "sprintReplicateOnRollover": sprint_replicate_on_rollover,
                "sprintNameFmt": sprint_name_fmt,
                "standupRecurrence": standup_recurrence,
                "standupRecursNextAt": standup_recurs_next_at,
                "changelogRecurrence": changelog_recurrence,
                "changelogRecursNextAt": changelog_recurs_next_at,
            }
        )
        if updated_by_client_duid is not UNSET:
            field_dict["updatedByClientDuid"] = updated_by_client_duid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        duid = d.pop("duid")

        def _parse_drafter_duid(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        drafter_duid = _parse_drafter_duid(d.pop("drafterDuid"))

        kind = SpaceKind(d.pop("kind"))

        accessible_by_team = d.pop("accessibleByTeam")

        accessible_by_user_duids = cast(List[str], d.pop("accessibleByUserDuids"))

        order = d.pop("order")

        title = d.pop("title")

        abrev = d.pop("abrev")

        description = d.pop("description")

        icon_kind = IconKind(d.pop("iconKind"))

        icon_name_or_emoji = d.pop("iconNameOrEmoji")

        color_hex = d.pop("colorHex")

        sprint_mode = SprintMode(d.pop("sprintMode"))

        sprint_replicate_on_rollover = d.pop("sprintReplicateOnRollover")

        sprint_name_fmt = d.pop("sprintNameFmt")

        def _parse_standup_recurrence(data: object) -> Union[Any, None]:
            if data is None:
                return data
            return cast(Union[Any, None], data)

        standup_recurrence = _parse_standup_recurrence(d.pop("standupRecurrence"))

        def _parse_standup_recurs_next_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                standup_recurs_next_at_type_0 = isoparse(data)

                return standup_recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        standup_recurs_next_at = _parse_standup_recurs_next_at(d.pop("standupRecursNextAt"))

        def _parse_changelog_recurrence(data: object) -> Union[Any, None]:
            if data is None:
                return data
            return cast(Union[Any, None], data)

        changelog_recurrence = _parse_changelog_recurrence(d.pop("changelogRecurrence"))

        def _parse_changelog_recurs_next_at(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                changelog_recurs_next_at_type_0 = isoparse(data)

                return changelog_recurs_next_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        changelog_recurs_next_at = _parse_changelog_recurs_next_at(d.pop("changelogRecursNextAt"))

        def _parse_updated_by_client_duid(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        updated_by_client_duid = _parse_updated_by_client_duid(d.pop("updatedByClientDuid", UNSET))

        space = cls(
            duid=duid,
            drafter_duid=drafter_duid,
            kind=kind,
            accessible_by_team=accessible_by_team,
            accessible_by_user_duids=accessible_by_user_duids,
            order=order,
            title=title,
            abrev=abrev,
            description=description,
            icon_kind=icon_kind,
            icon_name_or_emoji=icon_name_or_emoji,
            color_hex=color_hex,
            sprint_mode=sprint_mode,
            sprint_replicate_on_rollover=sprint_replicate_on_rollover,
            sprint_name_fmt=sprint_name_fmt,
            standup_recurrence=standup_recurrence,
            standup_recurs_next_at=standup_recurs_next_at,
            changelog_recurrence=changelog_recurrence,
            changelog_recurs_next_at=changelog_recurs_next_at,
            updated_by_client_duid=updated_by_client_duid,
        )

        space.additional_properties = d
        return space

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.saml_config_tenant_extension_status import SamlConfigTenantExtensionStatus

T = TypeVar("T", bound="SamlConfig")


@_attrs_define
class SamlConfig:
    """
    Attributes:
        status (SamlConfigTenantExtensionStatus): * `Disabled` - DISABLED
            * `Enabled` - ENABLED
        idp_url (str):
        idp_xml (str):
    """

    status: SamlConfigTenantExtensionStatus
    idp_url: str
    idp_xml: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        idp_url = self.idp_url

        idp_xml = self.idp_xml

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "idpUrl": idp_url,
                "idpXml": idp_xml,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = SamlConfigTenantExtensionStatus(d.pop("status"))

        idp_url = d.pop("idpUrl")

        idp_xml = d.pop("idpXml")

        saml_config = cls(
            status=status,
            idp_url=idp_url,
            idp_xml=idp_xml,
        )

        saml_config.additional_properties = d
        return saml_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
